@@ -28,6 +28,7 @@
  */
 
 import java.io.InputStream;
+import java.io.IOError;
 import java.io.IOException;
 import java.io.DataInputStream;
 import java.nio.file.DirectoryStream;
@@ -243,22 +244,30 @@ public class Basic {
         }
     }
 
-    @DataProvider(name = "topLevelPkgDirs")
-    private Object[][] topLevelPkgDirs() {
+    @DataProvider(name = "topLevelNonExistingDirs")
+    private Object[][] topLevelNonExistingDirs() {
         return new Object[][] {
             { "/java/lang" },
             { "java/lang"  },
             { "/java/util" },
             { "java/util"  },
+            { "/modules/modules"  },
+            { "/modules/modules/"  },
+            { "/modules/modules/java.base"  },
+            { "/modules/modules/java.base/"  },
+            { "/modules/modules/java.base/java/lang/Object.class"  },
+            { "/modules/modules/javax.scripting"  },
+            { "/modules/modules/javax.scripting/"  },
+            { "/modules/modules/javax.scripting/javax/script/ScriptEngine.class"  },
         };
     }
 
-    @Test(dataProvider = "topLevelPkgDirs")
+    @Test(dataProvider = "topLevelNonExistingDirs")
     public void testNotExists(String path) throws Exception {
         FileSystem fs = FileSystems.getFileSystem(URI.create("jrt:/"));
         Path dir = fs.getPath(path);
 
-        // package directories should not be there at top level
+        // These directories should not be there at top level
         assertTrue(Files.notExists(dir));
     }
 
@@ -271,17 +280,60 @@ public class Basic {
         Path top = fs.getPath("/");
         try (Stream<Path> stream = Files.walk(top)) {
             stream.forEach(path -> {
-                URI u = path.toUri();
+                String pathStr = path.toAbsolutePath().toString();
+                URI u = null;
+                try {
+                    u = path.toUri();
+                } catch (IOError e) {
+                    assertFalse(pathStr.startsWith("/modules"));
+                    return;
+                }
+
                 assertTrue(u.getScheme().equalsIgnoreCase("jrt"));
                 assertFalse(u.isOpaque());
                 assertTrue(u.getAuthority() == null);
-                assertEquals(u.getPath(), path.toAbsolutePath().toString());
+
+                pathStr = pathStr.substring("/modules".length());
+                if (pathStr.isEmpty()) {
+                    pathStr = "/";
+                }
+                assertEquals(u.getPath(), pathStr);
                 Path p = Paths.get(u);
                 assertEquals(p, path);
             });
         }
     }
 
+    // @bug 8216553: JrtFIleSystemProvider getPath(URI) omits /modules element from file path
+    @Test
+    public void testPathToURIConversion() throws Exception {
+        var uri = URI.create("jrt:/java.base/module-info.class");
+        var path = Path.of(uri);
+        assertTrue(Files.exists(path));
+
+        uri = URI.create("jrt:/java.base/../java.base/module-info.class");
+        boolean seenIAE = false;
+        try {
+            Path.of(uri);
+        } catch (IllegalArgumentException iaExp) {
+            seenIAE = true;
+        }
+        assertTrue(seenIAE);
+
+        // check round-trip
+        var jrtfs = FileSystems.getFileSystem(URI.create("jrt:/"));
+        assertTrue(Files.exists(jrtfs.getPath(path.toString())));
+
+        path = jrtfs.getPath("/modules/../modules/java.base/");
+        boolean seenIOError = false;
+        try {
+            path.toUri();
+        } catch (IOError ioError) {
+            seenIOError = true;
+        }
+        assertTrue(seenIOError);
+    }
+
     @Test
     public void testDirectoryNames() throws Exception {
         FileSystem fs = FileSystems.getFileSystem(URI.create("jrt:/"));
@@ -563,7 +615,6 @@ public class Basic {
             { "java.lang" },
             { "java.util" },
             { "java.nio"  },
-            { "jdk.nashorn.api.scripting" }
         };
     }
 
@@ -588,7 +639,6 @@ public class Basic {
             { "/packages/javax/java.xml"  },
             { "/packages/javax/java.management"  },
             { "/packages/java.util/java.base" },
-            { "/packages/jdk.nashorn.api.scripting/jdk.scripting.nashorn" },
         };
     }
 
@@ -607,7 +657,6 @@ public class Basic {
         return new Object[][] {
             { "java.base" },
             { "java.sql" },
-            { "jdk.scripting.nashorn" },
         };
     }
 
@@ -624,7 +673,6 @@ public class Basic {
         return new Object[][] {
             { "/modules/java.base/java/lang" },
             { "/modules/java.base/java/util/Vector.class" },
-            { "/modules/jdk.scripting.nashorn/jdk/nashorn" },
             { "/packages/java.lang/java.base/java/lang" },
             { "/packages/java.util/java.base/java/util/Vector.class" },
         };
@@ -716,5 +764,15 @@ public class Basic {
 
         assertTrue(Files.size(classFile) > 0L);
     }
+
+    // @bug 8266291: (jrtfs) Calling Files.exists may break the JRT filesystem
+    @Test
+    public void fileExistsCallBreaksFileSystem() throws Exception {
+        Path p = FileSystems.getFileSystem(URI.create("jrt:/")).getPath("modules");
+        boolean wasDirectory = Files.isDirectory(p);
+        Path m = p.resolve("modules");
+        Files.exists(m);
+        assertTrue(wasDirectory == Files.isDirectory(p));
+    }
 }
 
